
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <math.h>
#include <string>
#include "locator.h"
#include "c3ga_util.h"

locator::locator() : m_xf(1.0f), m_xfi(1.0f), m_selected(false) {
}

locator::locator(const c3ga &xf) : m_xf(xf), m_xfi(xf.inverse()), m_selected(false){
}

locator::locator(const locator &l) : m_xf(l.m_xf), m_xfi(l.m_xfi) , m_selected(l.m_selected) {

}

locator::~locator() {
}


locator &locator::operator=(const locator &l) {
	if (this != &l) {
		m_xf =l.m_xf;
		m_xfi = l.m_xfi;
		m_selected = l.m_selected;
	}
	return *this;
}

void locator::set_xf(const c3ga &xf) {
	m_xf = xf;// / sqrt(fabs(xf.norm_a())); // DIVISION == TEMP???
	m_xfi = xf.inverse();
}

void locator::pre_mul_xf(const c3ga &xf) {
	c3ga new_xf = xf * m_xf;
	set_xf(new_xf);
}

void locator::post_mul_xf(const c3ga &xf) {
	c3ga new_xf = m_xf * xf;

	set_xf(new_xf);
}

void locator::apply_transform_OpenGL() const {
	unit_versor_glMult(get_xf());
}

c3ga locator::get_translator() const {
	// subtract the transform flat origin from the regular flat origin, exponentiate, done:
	return (0.5 * ((c3ga::no ^ c3ga::ni) - (get_xf() * (c3ga::no ^ c3ga::ni) * get_xf().inverse()))).exp();
}

bool locator::can_scale() const {
	return true;
}

std::string locator::to_string() const {
	return std::string("[") + std::string(get_xf().string("%10.20e")) + std::string("]");
}





