
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _LOCATOR_H_
#define _LOCATOR_H_

#include "classes.h"

class locator {
public:
	locator();
	locator(const c3ga::EXForm &xf);
	locator(const locator &l);
	virtual ~locator();

	locator &operator=(const locator &l);

	void set_xf(const c3ga::EXForm &xf); ///< sets m_xf (and m_xfi)

	inline const c3ga::EXForm &get_xf() const {return m_xf;} ///< returns m_xf
	inline const c3ga::EXForm &get_xfi() const {return m_xfi;} ///< returns m_xfi
	inline const c3ga::om &get_M() const {return m_M;} ///< returns m_M
	inline const c3ga::om &get_Mi() const {return m_Mi;} ///< returns m_Mi

	/// returns position of locator as unit flat point
	inline const c3ga::normalizedFlatPoint &get_position() const { 
		return m_position;
	}

	inline void compute_position() { 
		m_position = c3ga::_normalizedFlatPoint(get_xf() * c3ga::noni * get_xfi());
	}

	/// transform a versor \param V according to this transfomation:
	inline c3ga::mv apply_xf_to(const c3ga::mv &V) const { 
		return (get_xf() * V * get_xfi());
	}

	/// transform a versor \param V according to this transfomation:
	inline c3ga::mv apply_xfi_to(const c3ga::mv &V) const { 
		return (get_xfi() * V * get_xf());
	}

	void pre_mul_xf(const c3ga::EXForm &xf); ///< m_xf = xf * m_xf
	void pre_mul_xf(const c3ga::rotor &xf); ///< m_xf = xf * m_xf
	void pre_mul_xf(const c3ga::translator &xf); ///< m_xf = xf * m_xf
	void pre_mul_xf(const c3ga::scalor &xf); ///< m_xf = xf * m_xf

	void post_mul_xf(const c3ga::EXForm &xf); ///< m_xf = m_xf * xf;
	void post_mul_xf(const c3ga::rotor &xf); ///< m_xf = m_xf * xf;
	void post_mul_xf(const c3ga::translator &xf); ///< m_xf = m_xf * xf;
	void post_mul_xf(const c3ga::scalor &xf); ///< m_xf = m_xf * xf;


	virtual bool can_scale() const; // should return true if this locator can scale

	/// multiplies the transform with the current GL matrix
	void apply_transform_OpenGL() const;

	bool get_selected() const {return m_selected;}
	void set_selected(bool s) {m_selected = s;}

	/// returns just the translation part of the transform
	c3ga::translator get_translator() const; 

	/// Turns this object into a string (e.g., for storing a scene to file)
	std::string to_string() const;

private:

	/// transform & inverse transform (CGA versor)
	c3ga::EXForm m_xf, m_xfi;	 

	c3ga::om m_M, m_Mi;
	void updateM();

	c3ga::normalizedFlatPoint m_position; // position of this location (precomputed for use during raytracing)

	bool m_selected; /// set to true when this locator is selected by the UI
};


#endif /* _LOCATOR_H_ */

