
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _RENDER_WINDOW_H_
#define _RENDER_WINDOW_H_

#include <vector>
#include <string>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Image.H>
#include <FL/Fl_Value_Slider.H>

#include "classes.h"

/**
This window display model & light source properties that can be edited.
It has two modes, one for models, one for light sources.
Toggle by calling use set_X_active().
*/
class render_window : public Fl_Double_Window {
public:
	render_window(int w, int h, const char* label=0);
	render_window(int x, int y, int w, int h, const char* label=0);
	virtual ~render_window();

	/// called by rw_callback (in anonymous namespace)
	void callback(Fl_Widget *W, void *Data); 

	/// called by rw_timer_callback (in anonymous namespace)
	void timer_callback(); 

	///< returns true if currently rendering
	bool rendering() const; 

private:
	void setup_widgets(); /// Creates all the widgets

	Fl_Button *m_render_button;
	Fl_Button *m_stop_render_button;
	Fl_Button *m_save_render_button;

	Fl_Int_Input *m_width_input;
	Fl_Int_Input *m_height_input;

	Fl_Value_Slider *m_multisample_slider;
	Fl_Box *m_render_output_box;

	Fl_Image *m_image;

	/// RGB buffer for rendering (3 * m_render_buffer_width * m_render_buffer_height):
	std::vector<unsigned char> m_render_buffer; 

	std::vector<bool> m_scanline_rendered; /// contains true if scanline 'idx' has been rendered
	int m_nb_scanline_rendered; /// contains number of bits set to 'true' in m_scanline_rendered

	/// width of m_render_buffer
	int m_render_buffer_width;
	/// height of m_render_buffer
	int m_render_buffer_height;

	/// the line we are currently rendering (negative if not rendering? otherwise [0 .... m_render_buffer_height) )
	int m_render_scanline;
	int m_scanline_step; /// how much to add to m_render_scanline to get to the next scanline

	/// start rendering according to widgets (invoked by m_render_button)
	void start_render();

	/// stop rendering (invoked by m_stop_render_button, but also called automatically when render is done)
	void stop_render();

	/*Fl_Group *m_model_group; /// the group containing the modeling widgets
	Fl_Group *m_light_source_group; /// the group containing the light source widgets
	// ambient color (model):
	pw_color_chooser *m_model_ambient_color_chooser;
	// diffuse color (model):
	pw_color_chooser *m_model_diffuse_color_chooser;
	// specular color (model):
	pw_color_chooser *m_model_specular_color_chooser;

	Fl_Value_Slider *m_direct_lighting_slider;
	Fl_Value_Slider *m_reflection_slider;
	Fl_Value_Slider *m_refraction_slider;
	Fl_Value_Slider *m_refractive_index_slider;

	// ambient color (light source):
	pw_color_chooser *m_light_source_ambient_color_chooser;
	// diffuse color (light source):
	pw_color_chooser *m_light_source_diffuse_color_chooser;
	// specular color (light source):
	pw_color_chooser *m_light_source_specular_color_chooser;


	/// set the label of the window
	void set_label(const std::string &pw_label_str);
	*/

};

#endif /* _RENDER_WINDOW_H_ */
