
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef _SURFACE_POINT_H_
#define _SURFACE_POINT_H_

#include "c3ga.h"
#include "model.h"
#include "../shared.src/color.h"

/**
This class carries information about a specific
point on some model / mesh. The data is computed after
it has been determined that a ray intersects this
point.
*/
class surface_point {
public:
	inline surface_point() : m_valid(false), m_material_valid(false)  { ///< null constructor
	}
	~surface_point() {/*todo*/};

	/// assignment operator
	inline surface_point& operator=(const surface_point &sp) {
		if (this != &sp) {
			m_valid = sp.m_valid;
			if (m_valid) {
				m_pt = sp.m_pt;
				m_distance = sp.m_distance;
				m_distance_local = sp.m_distance_local;
				m_material_valid = sp.m_material_valid;

				// todo: add more members as required!!!!!!!!!!

				if (m_material_valid) {
					m_ambient_color = sp.m_ambient_color; 
					m_diffuse_color = sp.m_diffuse_color;
					m_specular_color = sp.m_specular_color;
					m_shininess = sp.m_shininess;
					m_direct_lighting_factor = sp.m_direct_lighting_factor;
					m_reflection_factor = sp.m_reflection_factor;
					m_refraction_factor = sp.m_refraction_factor;
					m_refractive_index = sp.m_refractive_index;
				}
			}
		}
		return *this;
	}

	/// sets m_valid
	inline void set_valid(bool v) {m_valid = v;}
	/// returns m_valid
	inline bool get_valid() const {return m_valid;}

	/// returns m_pt
	inline const c3ga::normalizedFlatPoint &get_pt() const {return m_pt;}
	/// sets m_pt
	inline void set_pt(const c3ga::normalizedFlatPoint &pt) {m_pt = pt;}

	/// returns m_pt_local
	inline const c3ga::normalizedFlatPoint &get_pt_local() const {return m_pt_local;}
	/// sets m_pt_local
	inline void set_pt_local(const c3ga::normalizedFlatPoint &pt) {m_pt_local = pt;}

	/// returns m_att
	inline const c3ga::freeBivector &get_att() const {return m_att;}
	/// sets m_att
	inline void set_att(const c3ga::freeBivector &att) {m_att = att;}

	/// returns m_att_local
	inline const c3ga::freeBivector &get_att_local() const {return m_att_local;}
	/// sets m_att_local
	inline void set_att_local(const c3ga::freeBivector &att) {m_att_local = att;}

	/// returns m_distance
	inline c3ga::mv::Float get_distance() const {return m_distance;}
	/// sets m_distance
	inline void set_distance(c3ga::mv::Float d) {m_distance = d;}

	/// returns m_distance_local
	inline c3ga::mv::Float get_distance_local() const {return m_distance_local;}
	/// sets m_distance_local
	inline void set_distance_local(c3ga::mv::Float d) {m_distance_local = d;}

	/// sets m_side
	inline void set_side(int side) {m_side = side;}
	/// returns m_side
	inline int get_side() const {return m_side;}

	/// returns m_model
	inline const model *get_model() const {return m_model;}
	/// sets m_model
	inline void set_model(const model *m) {m_model = m;}

	/// returns m_face_idx
	inline unsigned int get_face_idx() const {return m_face_idx;}
	/// sets m_face_idx
	inline void set_face_idx(unsigned int idx) {m_face_idx = idx;}

	/// returns m_tex_pt
	inline const c3ga::normalizedFlatPoint2D &get_tex_pt() const {return m_tex_pt;}
	/// sets m_tex_pt
	inline void set_tex_pt(const c3ga::normalizedFlatPoint2D &pt) {m_tex_pt = pt;}


	/// sets m_direct_lighting_factor
	inline void set_direct_lighting_factor(c3ga::mv::Float dlf) {m_direct_lighting_factor = dlf;}
	/// sets m_reflection_factor
	inline void set_reflection_factor(c3ga::mv::Float rf) {m_reflection_factor = rf;}
	/// sets m_refraction_factor
	inline void set_refraction_factor(c3ga::mv::Float rf) {m_refraction_factor = rf;}
	/// sets m_refractive_index
	inline void set_refractive_index(c3ga::mv::Float ri) {m_refractive_index = ri;}


	/// returns m_direct_lighting_factor
	inline c3ga::mv::Float get_direct_lighting_factor() const {return m_direct_lighting_factor;}
	/// returns m_reflection_factor
	inline c3ga::mv::Float get_reflection_factor() const {return m_reflection_factor;}
	/// returns m_refraction_factor
	inline c3ga::mv::Float get_refraction_factor() const {return m_refraction_factor;}
	/// returns m_refractive_index
	inline c3ga::mv::Float get_refractive_index() const {return m_refractive_index;}

	/// sets m_ambient_color
	void set_ambient_color(const color &c) {m_ambient_color = c;}
	/// sets m_diffuse_color
	void set_diffuse_color(const color &c) {m_diffuse_color = c;}
	/// sets m_specular_color
	void set_specular_color(const color &c) {m_specular_color = c;}

	/// sets m_shininess
	void set_shininess(c3ga::mv::Float s) {m_shininess = (s < 3.5f) ? 3.5f : s;}

	/// returns m_shininess
	inline c3ga::mv::Float get_shininess() const {return m_shininess;}

	/// returns m_ambient_color
	inline const color &get_ambient_color() const {return m_ambient_color;}
	/// returns m_diffuse_color
	inline const color &get_diffuse_color() const {return m_diffuse_color;}
	/// returns m_specular_color
	inline const color &get_specular_color() const {return m_specular_color;}


	/// Fills in all the details for this surface point (assumes that it is valid without a check!)
	inline void compute_details() {
		get_model()->compute_surface_point_details(*this);
	}

private:
	bool  m_valid; ///< true if this surface_point actually contain valid information
	c3ga::normalizedFlatPoint m_pt; ///< position of intersection (in global frame)
	c3ga::normalizedFlatPoint m_pt_local; ///< position of intersection (in model frame)

	c3ga::mv::Float m_distance; ///< distance from ray position to intersection (in global frame)
	c3ga::mv::Float m_distance_local; ///< distance from ray position to intersection (in model frame)

	c3ga::freeBivector m_att; ///< attitude of surface (in global frame)
	c3ga::freeBivector m_att_local; ///< attitude of surface (in model frame)

	const model *m_model; ///< the model the point comes from
	unsigned int m_face_idx; ///< index of the face the point comes from

	c3ga::normalizedFlatPoint2D m_tex_pt; ///< index into texture

	int m_side; // from which side did the ray come? -1 is back side of face, +1 is front side of face

	bool m_material_valid; ///< true if the material properties below contain valid information

	color m_ambient_color; ///< ambient color of this model (unless ambient texture is specified)
	color m_diffuse_color; ///< diffuse color of this model (unless diffuse texture is specified)
	color m_specular_color; ///< specular color of this model (unless specular texture is specified)

	c3ga::mv::Float m_shininess; ///< shininess of specular component

	c3ga::mv::Float m_direct_lighting_factor; ///< weight assigned to 'OpenGL' lighting [0 ... 1]
	c3ga::mv::Float m_reflection_factor; ///< weight assigned to reflections [0 ... 1]
	c3ga::mv::Float m_refraction_factor; ///< weight assigned to refractions [0 ... 1]

	c3ga::mv::Float m_refractive_index; ///<

};



#endif /* _SURFACE_POINT_H_ */

