
#include "texture.h"
#include "../shared.src/pngfe.h"
#include "../shared.src/string_util.h"

texture::texture() : m_enabled(false), m_valid(false), m_width(-1), m_height(-1), /*m_magnitude(1.0), */m_average_color(0.5, 0.5, 0.5) {
}

texture::texture(const std::string &filename) {
	load(filename);
}

texture::texture(const texture &t) : 
	m_enabled(t.m_enabled),
	m_valid(t.m_valid), m_filename(t.m_filename),
	m_width(t.m_width), m_height(t.m_height),
	m_average_color(t.m_average_color), m_pixel(t.m_pixel)/*,
	m_magnitude(t.m_magnitude)*/ {
}


texture::~texture() {

}

texture &texture::operator=(const texture &t) {
	if (this != &t) {
		m_enabled = t.m_enabled;
		m_valid = t.m_valid;
		m_filename = t.m_filename;
		m_width = t.m_width;
		m_height = t.m_height;
		m_average_color = t.m_average_color;
		m_pixel = t.m_pixel;
//		m_magnitude = t.m_magnitude;
	}
	return *this;
}


void texture::load(const std::string &filename) {
	m_filename = filename;
	m_valid = false;
	m_pixel.clear();
	m_width = m_height = -1;

	if (filename.size()) {
		// try to load png
		try {
			load_png(m_filename, NULL, &m_width, &m_height, PNG_NO_ALLOC);
			if ((m_width <= 0) || (m_height <= 0))
				throw std::string("texture::load(): invalid PNG image");
			m_pixel.resize(m_width * m_height * 3);

			unsigned char *pixel = &(m_pixel[0]);
			load_png(m_filename, &pixel, &m_width, &m_height, PNG_NO_ALLOC);

			m_valid = true;
		}
		catch (std::string &) {
			m_pixel.clear();
			m_width = m_height = -1;
			return;
		}
	}
	// compute average color
	double sum[3] = {0.0, 0.0, 0.0};
	for (unsigned int i = 0; i < m_pixel.size(); i += 3) {
		sum[0] += (double)m_pixel[i + 0];
		sum[1] += (double)m_pixel[i + 1];
		sum[2] += (double)m_pixel[i + 2];
	}
	m_average_color.set((float)(3.0 * sum[0]  / (256.0 * m_pixel.size())),
		(float)(3.0 * sum[1]  / (256.0 * m_pixel.size())),
		(float)(3.0 * sum[2]  / (256.0 * m_pixel.size())));
}

const std::string &texture::get_filename() const {
	return m_filename;
}

const color &texture::get_average_color() const {
	return m_average_color;
}

void texture::set_enabled(bool e) {
	m_enabled = e;
}


